@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.604-2-g1ebccb10
+package JSON::Schema::Modern; # git description: v0.610-15-g971fc746
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema using a JSON Schema
 # KEYWORDS: JSON Schema validator data validation structure specification
 
-our $VERSION = '0.605';
+our $VERSION = '0.611';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -17,9 +17,11 @@ use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
+no if "$]" >= 5.041009, feature => 'smartmatch';
+no feature 'switch';
 use Mojo::JSON ();  # for JSON_XS, MOJO_NO_JSON_XS environment variables
 use Carp qw(croak carp);
-use List::Util 1.55 qw(pairs first uniqint pairmap uniq any);
+use List::Util 1.55 qw(pairs first uniqint pairmap uniq any min);
 use Ref::Util 0.100 qw(is_ref is_plain_hashref);
 use builtin::compat qw(refaddr load_module);
 use Mojo::URL;
@@ -99,6 +101,9 @@ has [qw(collect_annotations scalarref_booleans stringy_numbers strict)] => (
   isa => Bool,
 );
 
+# Validation §7.1-2: "Note that the "type" keyword in this specification defines an "integer" type
+# which is not part of the data model. Therefore a format attribute can be limited to numbers, but
+# not specifically to integers."
 my $core_types = Enum[qw(null object array boolean string number)];
 my @core_formats = qw(date-time date time duration email idn-email hostname idn-hostname ipv4 ipv6 uri uri-reference iri iri-reference uuid uri-template json-pointer relative-json-pointer regex);
 
@@ -281,7 +286,7 @@ sub traverse ($self, $schema_reference, $config_override = {}) {
 
   # Note: the starting position is not guaranteed to be at the root of the $document,
   # nor is the fragment portion of this uri necessarily empty
-  my $initial_uri = Mojo::URL->new($config_override->{initial_schema_uri} // '');
+  my $initial_uri = Mojo::URL->new($config_override->{initial_schema_uri} // ());
   my $initial_path = $config_override->{traversed_schema_path} // '';
   my $spec_version = $config_override->{specification_version} // $self->specification_version // SPECIFICATION_VERSION_DEFAULT;
 
@@ -300,6 +305,7 @@ sub traverse ($self, $schema_reference, $config_override = {}) {
     initial_schema_uri => $initial_uri,     # the canonical URI as of the start of this method or last $id
     traversed_schema_path => $initial_path, # the accumulated traversal path as of the start or last $id
     schema_path => '',                      # the rest of the path, since the start of this method or last $id
+    spec_version => $spec_version,
     errors => [],
     identifiers => {},
     subschemas => [],
@@ -309,20 +315,32 @@ sub traverse ($self, $schema_reference, $config_override = {}) {
     traverse => 1,
   };
 
+  my $valid = 1;
+
   try {
-    my $for_canonical_uri = Mojo::URL->new(
-      (is_plain_hashref($schema_reference) && exists $schema_reference->{'$id'}
-          ? Mojo::URL->new($schema_reference->{'$id'}) : undef)
-        // $state->{initial_schema_uri});
-    $for_canonical_uri->fragment(undef) if not length $for_canonical_uri->fragment;
+    # determine the initial value of spec_version and vocabularies, so we have something to start
+    # with in _traverse_subschema().
+    # a subsequent "$schema" keyword can still change these values, and it is always processed
+    # first, so the override is skipped if the keyword exists in the schema
+    $state->{metaschema_uri} =
+      (is_plain_hashref($schema_reference) && exists $schema_reference->{'$schema'} ? undef
+        : $config_override->{metaschema_uri}) // $self->METASCHEMA_URIS->{$spec_version};
 
-    # a subsequent "$schema" keyword can still change these values
-    $state->@{qw(spec_version vocabularies)} = $self->_get_metaschema_info(
-      $config_override->{metaschema_uri} // $self->METASCHEMA_URIS->{$spec_version},
-      $for_canonical_uri,
-    );
+    if (my $metaschema_info = $self->_get_metaschema_vocabulary_classes($state->{metaschema_uri})) {
+      $state->@{qw(spec_version vocabularies)} = @$metaschema_info;
+    }
+    else {
+      # metaschema has not been processed for vocabularies yet...
 
-    my $valid = $self->_traverse_subschema($schema_reference, $state);
+      die 'something went wrong - cannot get metaschema data for '.$state->{metaschema_uri}
+        if not $config_override->{metaschema_uri};
+
+      # use the Core vocabulary to set metaschema info via the '$schema' keyword implementation
+      $valid = $self->_get_metaschema_vocabulary_classes($self->METASCHEMA_URIS->{$spec_version})->[1][0]
+          ->_traverse_keyword_schema({ '$schema' => $state->{metaschema_uri}.'' }, $state);
+    }
+
+    $valid = $self->_traverse_subschema($schema_reference, $state) if $valid and not $state->{errors}->@*;
     die 'result is false but there are no errors' if not $valid and not $state->{errors}->@*;
     die 'result is true but there are errors' if $valid and $state->{errors}->@*;
   }
@@ -371,30 +389,17 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
 
   my $valid;
   try {
-    my $schema_info;
+    # traverse is called via add_schema -> ::Document->new -> ::Document->BUILD
+    $schema_reference = $self->add_schema($schema_reference)->canonical_uri
+      if is_ref($schema_reference) and not $schema_reference->$_isa('Mojo::URL');
 
-    if (not is_ref($schema_reference) or $schema_reference->$_isa('Mojo::URL')) {
-      $schema_info = $self->_fetch_from_uri($schema_reference);
-    }
-    else {
-      # traverse is called via add_schema -> ::Document->new -> ::Document->BUILD
-      my $document = $self->add_schema($schema_reference);
-      my $base_resource = $document->_get_resource($document->canonical_uri)
-        || croak "couldn't get resource: document parse error";
-
-      $schema_info = {
-        schema => $document->schema,
-        document => $document,
-        document_path => '',
-        $base_resource->%{qw(canonical_uri specification_version vocabularies configs)},
-      };
-    }
+    my $schema_info = $self->_fetch_from_uri($schema_reference);
+    abort($state, 'EXCEPTION: unable to find resource "%s"', $schema_reference)
+      if not $schema_info;
 
     abort($state, 'EXCEPTION: collect_annotations cannot be used with specification_version '.$schema_info->{specification_version})
       if $config_override->{collect_annotations} and $schema_info->{specification_version} =~ /^draft[467]$/;
 
-    abort($state, 'EXCEPTION: unable to find resource "%s"', $schema_reference)
-      if not $schema_info;
     abort($state, 'EXCEPTION: "%s" is not a schema', $schema_reference)
       if not $schema_info->{document}->get_entity_at_location($schema_info->{document_path});
 
@@ -470,8 +475,19 @@ sub validate_schema ($self, $schema, $config_override = {}) {
   my $metaschema_uri = is_plain_hashref($schema) && $schema->{'$schema'} ? $schema->{'$schema'}
     : $self->METASCHEMA_URIS->{$self->specification_version // $self->SPECIFICATION_VERSION_DEFAULT};
 
-  return $self->evaluate($schema, $metaschema_uri,
+  my $result = $self->evaluate($schema, $metaschema_uri,
     { %$config_override, $self->strict || $config_override->{strict} ? (_strict_schema_data => 1) : () });
+
+  return $result if not $result->valid;
+
+  my $state = $self->traverse($schema, $config_override);
+  return JSON::Schema::Modern::Result->new(
+    output_format => $self->output_format,
+    valid => 0,
+    errors => $state->{errors},
+  ) if $state->{errors}->@*;
+
+  return $result; # valid: true
 }
 
 sub get ($self, $uri_reference) {
@@ -605,7 +621,7 @@ sub _traverse_subschema ($self, $schema, $state) {
         $valid = 0;
         next;
       }
-      warn 'traverse result is true but there are errors (keyword: '.$keyword.')'
+      warn 'traverse result is true but there are errors ('.$keyword.': '.$state->{errors}[-1]->error
         if $error_count != $state->{errors}->@*;
 
       # a keyword changed the keyword list for this vocabulary; re-fetch the list before continuing
@@ -669,7 +685,7 @@ sub _eval_subschema ($self, $data, $schema, $state) {
 
   return 1 if not keys %$schema;
 
-  # find all schema locations in effect at this data path + canonical_uri combination
+  # find all schema locations in effect at this data path + uri combination
   # if any of them are absolute prefix of this schema location, we are in a loop.
   my $canonical_uri = canonical_uri($state);
   my $schema_location = $state->{traversed_schema_path}.$state->{schema_path};
@@ -809,7 +825,7 @@ my $path_type = Str->where('m{^(?:/|$)}');  # JSON pointer relative to the docum
 has _resource_index => (
   is => 'bare',
   isa => Map[my $resource_key_type = Str->where('!/#/'), my $resource_type = Dict[
-      # always stringwise-equal to the top level key
+      # may not be stringwise-equal to the top level key
       canonical_uri => (InstanceOf['Mojo::URL'])->where(q{not defined $_->fragment}),
       path => $path_type,
       specification_version => my $spec_version_type = Enum(SPECIFICATION_VERSIONS_SUPPORTED),
@@ -927,41 +943,6 @@ sub _get_metaschema_vocabulary_classes { $_[0]->__metaschema_vocabulary_classes-
 sub _set_metaschema_vocabulary_classes { $_[0]->__metaschema_vocabulary_classes->{$_[1] =~ s/#$//r} = $mvc_type->($_[2]) }
 sub __all_metaschema_vocabulary_classes { values $_[0]->__metaschema_vocabulary_classes->%* }
 
-# retrieves metaschema info either from cache or by parsing the schema for vocabularies
-# throws a JSON::Schema::Modern::Result on error
-sub _get_metaschema_info ($self, $metaschema_uri, $for_canonical_uri) {
-  # check the cache. specification metaschemas are already populated.
-  my $metaschema_info = $self->_get_metaschema_vocabulary_classes($metaschema_uri);
-  return @$metaschema_info if $metaschema_info;
-
-  # otherwise, fetch the metaschema and parse its $vocabulary keyword.
-  # we do this by traversing a baby schema with just the $schema keyword.
-  my $state = $self->traverse({ '$schema' => $metaschema_uri.'' });
-  die JSON::Schema::Modern::Result->new(
-    output_format => $self->output_format,
-    valid => JSON::PP::false,
-    errors => [
-      map {
-        my $e = $_;
-        # absolute location is undef iff the location = '/$schema'
-        my $absolute_location = $e->absolute_keyword_location // $for_canonical_uri;
-        JSON::Schema::Modern::Error->new(
-          depth => $e->depth,
-          mode => 'traverse',
-          keyword => $e->keyword eq '$schema' ? '' : $e->keyword,
-          instance_location => $e->instance_location,
-          keyword_location => ($for_canonical_uri->fragment//'').($e->keyword_location =~ s{^/\$schema\b}{}r),
-          length $absolute_location ? ( absolute_keyword_location => $absolute_location ) : (),
-          error => $e->error,
-        )
-      }
-      $state->{errors}->@* ],
-    exception => 1,
-  ) if $state->{errors}->@*;
-
-  return ($state->{spec_version}, $state->{vocabularies});
-}
-
 # translate vocabulary URIs into classes, caching the results (if any)
 sub _fetch_vocabulary_data ($self, $state, $schema_info) {
   if (not exists $schema_info->{schema}{'$vocabulary'}) {
@@ -972,6 +953,8 @@ sub _fetch_vocabulary_data ($self, $state, $schema_info) {
   }
 
   my $valid = 1;
+  # Core §8.1.2-6: "The "$vocabulary" keyword SHOULD be used in the root schema of any schema
+  # document intended for use as a meta-schema. It MUST NOT appear in subschemas."
   $valid = E($state, '$vocabulary can only appear at the document root') if length $schema_info->{document_path};
   $valid = E($state, 'metaschemas must have an $id') if not exists $schema_info->{schema}{'$id'};
 
@@ -1007,12 +990,13 @@ sub _fetch_vocabulary_data ($self, $state, $schema_info) {
 }
 
 # used for determining a default '$schema' keyword where there is none
+# these are also normalized as this is how we cache them
 use constant METASCHEMA_URIS => {
   'draft2020-12' => 'https://json-schema.org/draft/2020-12/schema',
   'draft2019-09' => 'https://json-schema.org/draft/2019-09/schema',
-  'draft7' => 'http://json-schema.org/draft-07/schema#',
-  'draft6' => 'http://json-schema.org/draft-06/schema#',
-  'draft4' => 'http://json-schema.org/draft-04/schema#',
+  'draft7' => 'http://json-schema.org/draft-07/schema',
+  'draft6' => 'http://json-schema.org/draft-06/schema',
+  'draft4' => 'http://json-schema.org/draft-04/schema',
 };
 
 use constant CACHED_METASCHEMAS => {
@@ -1086,21 +1070,43 @@ sub _get_or_load_resource ($self, $uri) {
 # creates a Document and adds it to the resource index, if not already present.
 sub _fetch_from_uri ($self, $uri_reference) {
   $uri_reference = Mojo::URL->new($uri_reference) if not is_ref($uri_reference);
-  my $fragment = $uri_reference->fragment;
 
+  # this is *a* resource that would contain our desired location, but may not be the closest one
   my $resource = $self->_get_or_load_resource($uri_reference->clone->fragment(undef));
   return if not $resource;
 
+  my $fragment = $uri_reference->fragment;
   if (not length($fragment) or $fragment =~ m{^/}) {
     my $subschema = $resource->{document}->get(my $document_path = $resource->{path}.($fragment//''));
     return if not defined $subschema;
-    my $doc_addr = refaddr($resource->{document});
-    my $closest_resource = first { !length($_->[1]{path})       # document root
-        || length($document_path)
-          && $document_path =~ m{^\Q$_->[1]{path}\E(?:/|\z)} }  # path is above present location
-      sort { length($b->[1]{path}) <=> length($a->[1]{path}) }  # sort by length, descending
-      grep { refaddr($_->[1]{document}) == $doc_addr }          # in same document
-      $self->_resource_pairs;
+
+    my $closest_resource;
+    if (not length $fragment) { # we already have the canonical resource root
+      $closest_resource = [ undef, $resource ];
+    }
+    else {
+      # determine the canonical uri by finding the closest schema resource(s)
+      my $doc_addr = refaddr($resource->{document});
+      my @closest_resources =
+        sort { length($b->[1]{path}) <=> length($a->[1]{path}) }  # sort by length, descending
+        grep { !length($_->[1]{path})       # document root
+          || length($document_path)
+            && $document_path =~ m{^\Q$_->[1]{path}\E(?:/|\z)} }  # path is above desired location
+        grep { refaddr($_->[1]{document}) == $doc_addr }          # in same document
+        $self->_resource_pairs;
+
+      # now whittle down to all the resources with the same document path as the first candidate
+      if (@closest_resources > 1) {
+        # find the resource key that most closely matches the original query uri, by matching prefixes
+        my $match = $uri_reference.'';
+        @closest_resources =
+          sort { _prefix_match_length($b->[0], $match) <=> _prefix_match_length($a->[0], $match) }
+          grep $_->[1]{path} eq $closest_resources[0]->[1]{path},
+          @closest_resources;
+      }
+
+      $closest_resource = $closest_resources[0];
+    }
 
     my $canonical_uri = $closest_resource->[1]{canonical_uri}->clone
       ->fragment(substr($document_path, length($closest_resource->[1]{path})));
@@ -1110,20 +1116,30 @@ sub _fetch_from_uri ($self, $uri_reference) {
       schema => $subschema,
       canonical_uri => $canonical_uri,
       document_path => $document_path,
-      $resource->%{qw(document specification_version vocabularies configs)}, # reference, not copy
+      $closest_resource->[1]->%{qw(document specification_version vocabularies configs)}, # reference, not copy
     };
   }
   else {  # we are following a URI with a plain-name fragment
     return if not my $subresource = ($resource->{anchors}//{})->{$fragment};
     return {
       schema => $resource->{document}->get($subresource->{path}),
-      canonical_uri => $subresource->{canonical_uri}->clone, # this is *not* the anchor-containing URI
+      canonical_uri => $subresource->{canonical_uri}, # this is *not* the anchor-containing URI
       document_path => $subresource->{path},
       $resource->%{qw(document specification_version vocabularies configs)}, # reference, not copy
     };
   }
 }
 
+# given two strings, determines the number of characters in common, starting from the first
+# character
+sub _prefix_match_length ($x, $y) {
+  my $len = min(length($x), length($y));
+  foreach my $pos (0..$len) {
+    return $pos if substr($x, $pos, 1) ne substr($y, $pos, 1);
+  }
+  return $len;
+}
+
 # Mojo::JSON::JSON_XS is false when the environment variable $MOJO_NO_JSON_XS is set
 # and also checks if Cpanel::JSON::XS is installed.
 # Mojo::JSON falls back to its own pure-perl encoder/decoder but does not support all the options
@@ -1248,7 +1264,7 @@ JSON::Schema::Modern - Validate data against a schema using a JSON Schema
 
 =head1 VERSION
 
-version 0.605
+version 0.611
 
 =head1 SYNOPSIS
 
@@ -1349,8 +1365,8 @@ Overrides to existing formats (see L</Format Validation>)
 must be specified in the form of C<< { $format_name => $format_sub } >>, where
 the format sub is a subref that takes one argument and returns a boolean result. New formats must
 be specified in the form of C<< { $format_name => { type => $type, sub => $format_sub } } >>,
-where the type indicates which of the core JSON Schema types (null, object, array, boolean, string,
-number, or integer) the instance value must be for the format validation to be considered.
+where the type indicates which of the data model types (null, object, array, boolean, string,
+or number) the instance value must be for the format validation to be considered.
 
 =head2 validate_content_schemas
 
@@ -1454,6 +1470,7 @@ Defaults to false.
 =head1 METHODS
 
 =for Pod::Coverage BUILDARGS FREEZE THAW
+CACHED_METASCHEMAS METASCHEMA_URIS SPECIFICATION_VERSIONS_SUPPORTED SPECIFICATION_VERSION_DEFAULT
 
 =head2 evaluate_json_string
 
@@ -1475,11 +1492,7 @@ a Perl data structure, such as what is returned from a JSON decode operation,
 
 =item *
 
-a L<JSON::Schema::Modern::Document> object,
-
-=item *
-
-or a URI string indicating the location where such a schema is located.
+or a URI string indicating the identity of such a schema.
 
 =back
 
@@ -1507,7 +1520,7 @@ C<initial_schema_uri>: adjusts the recorded absolute keyword location as of the
 
 =item *
 
-C<effective_base_uri>: locations in errors and annotations are resolved against this URI
+C<effective_base_uri>: locations in errors and annotations are resolved against this URI (only useful when providing an inline schema that does not declare an absolute base URI for itself)
 
 =back
 
@@ -1533,11 +1546,7 @@ a Perl data structure, such as what is returned from a JSON decode operation,
 
 =item *
 
-a L<JSON::Schema::Modern::Document> object,
-
-=item *
-
-or a URI string indicating the location where such a schema is located.
+or a URI string indicating the identity of such a schema.
 
 =back
 
@@ -1561,7 +1570,7 @@ C<traversed_schema_path>: adjusts the accumulated path as of the start of evalua
 
 =item *
 
-C<effective_base_uri>: locations in errors and annotations are resolved against this URI
+C<effective_base_uri>: locations in errors and annotations are resolved against this URI (only useful when providing an inline schema that does not declare an absolute base URI for itself)
 
 =back
 
@@ -1656,24 +1665,28 @@ identifier (either provided explicitly in the method call, or via an C<$id> keyw
 root), all such previous schemas are removed from memory and can no longer be referenced.
 
 If there were errors in the document, will die with these errors;
-otherwise returns the L<JSON::Schema::Modern::Document> that contains the added schema.
+otherwise returns the L<JSON::Schema::Modern::Document> that contains the added schema. URIs
+identified within this document will not be resolved to the provided C<$uri> argument, so you can
+re-add the document object again (with L</add_document>, below) using a new base URI if you wish.
 
 =head2 add_document
 
   $js->add_document($uri => $document);
   $js->add_document($document);
 
-Introduces the L<JSON::Schema::Modern::Document> (or subclass)
-object, representing a JSON Schema, to the implementation, registering it under the indicated URI if
-provided (and all known identifiers within the document will be added as well, resolved against the
-provided URI).
+Makes the L<JSON::Schema::Modern::Document> (or subclass)
+object, representing a JSON Schema, available to the evaluator. All identifiers known to the
+document are added to the evaluator's resource index; if the C<$uri> argument is provided, those
+identifiers are resolved against C<$uri> as they are added.
 
-If you add multiple documents (either with this method, or implicitly via L</evaluate>) with no root
+C<$uri> itself is also added to the resource index, referencing the root of the document itself.
+
+If you add multiple documents (either with this method, or implicitly via C</add_schema> or L</evaluate>) with no root
 identifier (either provided explicitly in the method call, or via an C<$id> keyword at the schema
 root), all such previous schemas are removed from memory and can no longer be referenced.
 
-If there were errors in the document, will die with these errors;
-otherwise returns the L<JSON::Schema::Modern::Document> object.
+If there were errors in the document, this method will die with these errors;
+otherwise it returns the L<JSON::Schema::Modern::Document> object.
 
 =head2 add_format_validation
 
@@ -1893,7 +1906,8 @@ By default (and unless you specify a custom metaschema with the C<$schema> keywo
 L<JSON::Schema::Modern::Document/metaschema>),
 formats are treated only as annotations, not assertions. When L</validate_formats> is
 true, strings are also checked against the format as specified in the schema. At present the
-following formats are supported (use of any other formats than these will always evaluate as true,
+following formats are supported for the latest version of the specification
+(use of any other formats than these will always evaluate as true,
 but remember you can always supply custom format handlers; see L</format_validations> above):
 
 =over 4
@@ -1969,13 +1983,18 @@ C<regex>
 =back
 
 A few optional prerequisites are needed for some of these (if the prerequisite is missing,
-validation will always succeed):
+validation will always succeed, unless draft2020-12 is in use with the Format-Assertion vocabulary
+declared in the metaschema, in which case use of the format will produce an error).
 
 =over 4
 
 =item *
 
-C<date-time>, C<date>, and C<time> require L<Time::Moment>, L<DateTime::Format::RFC3339>
+C<date-time> and C<date> require L<Time::Moment>
+
+=item *
+
+C<date-time> also requires L<DateTime::Format::RFC3339>
 
 =item *
 
@@ -1983,20 +2002,20 @@ C<email> and C<idn-email> require L<Email::Address::XS> version 1.04 (or higher)
 
 =item *
 
-C<hostname> and C<idn-hostname> require L<Data::Validate::Domain>
+C<hostname> and C<idn-hostname> require L<Data::Validate::Domain> version 0.13 (or higher)
 
 =item *
 
-C<idn-hostname> requires L<Net::IDN::Encode>
+C<idn-hostname> also requires L<Net::IDN::Encode>
 
 =back
 
 =head2 Specification Compliance
 
 This implementation is now fully specification-compliant (for versions
-draft4, draft6, draft7, draft2019-09, draft2020-12),
-but until version 1.000 is released, it is
-still deemed to be missing some optional but quite useful features, such as:
+draft4, draft6, draft7, draft2019-09, draft2020-12).
+
+However, some potentially-useful features are not yet implemented, such as:
 
 =for stopwords Mojolicious
 
@@ -2016,7 +2035,7 @@ loading schema documents from a local web application (e.g. L<Mojolicious>)
 
 =item *
 
-additional output formats beyond C<flag>, C<basic>, and C<terse> (L<https://json-schema.org/draft/2020-12/json-schema-core.html#rfc.section.12>)
+additional "official" output formats beyond C<flag>, C<basic>, and C<terse> (L<https://json-schema.org/draft/2020-12/json-schema-core.html#rfc.section.12>)
 
 =back
 
@@ -2054,31 +2073,23 @@ L<RFC8259: The JavaScript Object Notation (JSON) Data Interchange Format|https:/
 
 =item *
 
-L<RFC3986: Uniform Resource Identifier (URI): Generic Syntax|https://datatracker.ietf.org/doc/html/rfc3986>
+L<RFC3986: Uniform Resource Identifier (URI): Generic Syntax|https://datatracker.ietf.org/doc/html/rfc3986> dependencies and faster evaluation
 
 =item *
 
-L<Test::JSON::Schema::Acceptance>: contains the official JSON Schema test suite
+L<https://json-schema.org/draft/2020-12>
 
 =item *
 
-L<JSON::Schema::Tiny>: a more stripped-down implementation of the specification, with fewer dependencies and faster evaluation
+L<https://json-schema.org/draft/2019-09>
 
 =item *
 
-L<https://json-schema.org/draft/2020-12/release-notes.html>
+L<https://json-schema.org/draft-07>
 
 =item *
 
-L<https://json-schema.org/draft/2019-09/release-notes.html>
-
-=item *
-
-L<https://json-schema.org/draft-07/json-schema-release-notes.html>
-
-=item *
-
-L<https://json-schema.org/draft-06#draft-06-release-notes>
+L<https://json-schema.org/draft-06>
 
 =item *
 
@@ -2090,6 +2101,18 @@ L<Understanding JSON Schema|https://json-schema.org/understanding-json-schema>:
 
 =item *
 
+L<Test::JSON::Schema>: test your data against a JSON Schema
+
+=item *
+
+L<Test::JSON::Schema::Acceptance>: contains the official JSON Schema test suite
+
+=item *
+
+L<JSON::Schema::Tiny>: a more stripped-down implementation of the specification, with fewer
+
+=item *
+
 L<OpenAPI::Modern>: a parser and evaluator for OpenAPI v3.1 documents
 
 =item *
@@ -2129,4 +2152,6 @@ This software is copyright (c) 2020 by Karen Etheridge.
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+Some schema files have their own licence, in share/LICENSE.
+
 =cut
